package com.reactlibrary;

import android.content.Context;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Matrix;
import android.graphics.Paint;

import androidx.exifinterface.media.ExifInterface;
import android.net.Uri;
import android.provider.MediaStore;

import android.util.Log;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;


public final class PictureUtils {

    //Scaling and maintain aspect ratio
    public static Bitmap getScaledBitmap(String path, int destWidth, int destHeight){
        BitmapFactory.Options options = new BitmapFactory.Options();
        options.inJustDecodeBounds = true;
        BitmapFactory.decodeFile(path, options);

        float srcWidth = options.outWidth;
        float srcHeight = options.outHeight;

        int sampleSize = 1;
        if(srcHeight > destHeight || srcWidth > destWidth){
            float heightScale = srcHeight / destHeight;
            float widthScale = srcWidth / destWidth;

            sampleSize = Math.round(heightScale > widthScale ? heightScale : widthScale);
        }

        options = new BitmapFactory.Options();
        options.inSampleSize = sampleSize;

        return BitmapFactory.decodeFile(path, options);
    }

    public static Bitmap getScaledBitmap(Context context, int res, int destWidth, int destHeight){
        BitmapFactory.Options options = new BitmapFactory.Options();
        options.inJustDecodeBounds = true;
        BitmapFactory.decodeResource(context.getResources(), res, options);

        float srcWidth = options.outWidth;
        float srcHeight = options.outHeight;

        int sampleSize = 1;
        if(srcHeight > destHeight || srcWidth > destWidth){
            float heightScale = srcHeight / destHeight;
            float widthScale = srcWidth / destWidth;

            sampleSize = Math.round(heightScale > widthScale ? heightScale : widthScale);
        }

        options = new BitmapFactory.Options();
        options.inSampleSize = sampleSize;
        Bitmap bmp = BitmapFactory.decodeResource(context.getResources(), res, options);
        return bmp;
    }

    public static Bitmap getCompressedBitmap(Bitmap imageBitmap, int compressRatio){
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        imageBitmap.compress(Bitmap.CompressFormat.JPEG, compressRatio, outputStream);

        return BitmapFactory.decodeStream(new ByteArrayInputStream(outputStream.toByteArray()));
    }

    public static String cacheImage(Context context, Bitmap imageBitmap, String name){
        File imageCacheFile = null;
        FileOutputStream outputStream = null;
        ByteArrayOutputStream stream = null;

        try{
            imageCacheFile = new File(context.getCacheDir(),  name + ".jpeg");
            stream = new ByteArrayOutputStream();
            outputStream = new FileOutputStream(imageCacheFile);
            imageBitmap.compress(Bitmap.CompressFormat.JPEG, 100, stream);
            outputStream.write(stream.toByteArray());
            outputStream.close();
            stream.close();
        } catch (FileNotFoundException e){
            e.printStackTrace();
        } catch (IOException e){
            e.printStackTrace();
        }

        return imageCacheFile.getPath();
    }

    public static Bitmap watermarkImage(Context context, Bitmap imageBitmap){
        Bitmap watermarkedBitmap = Bitmap.createBitmap(imageBitmap.getWidth(), imageBitmap.getHeight(), imageBitmap.getConfig());

        Canvas canvas = new Canvas(watermarkedBitmap);
        canvas.drawBitmap(imageBitmap, 0,0, null);

        int scaledWatermarkWidth = (int) Math.floor(watermarkedBitmap.getWidth() * (75.0/400));
        int scaledWatermarkHeight = (int) Math.floor(watermarkedBitmap.getHeight() * (45.0/400));

        Bitmap watermarkBitmap = getScaledBitmap(context, R.mipmap.aqar_logo_watermark, scaledWatermarkWidth, scaledWatermarkHeight);

        int differenceOfHeights = imageBitmap.getHeight() - scaledWatermarkHeight;
        int differenceOfWidths = imageBitmap.getWidth() - scaledWatermarkWidth;

        Random randomGenerator = new Random();
        int randomYPoint = randomGenerator.nextInt(differenceOfHeights + 1);
        int randomXPoint = randomGenerator.nextInt(differenceOfWidths + 1);

        canvas.drawBitmap(watermarkBitmap, randomXPoint, randomYPoint, null);

        return watermarkedBitmap;
    }

    public static List<String> processImages(Context context, int maxWidth, int maxHeight, int compressRatio, List<String> imagePaths, boolean doWatermark){
        List<String> processedImagesBitmapPaths = new ArrayList<>();

        for(String imagePath: imagePaths){
            //get the image name from the path (i.e IMG_1234.JPG)
            String[] imagePathParts = imagePath.split("/");
            String imageName = imagePathParts[imagePathParts.length-1];

            //Scale then compress image
            Bitmap bitmap = PictureUtils.getScaledBitmap(imagePath, maxWidth, maxHeight);
            bitmap = PictureUtils.getRotatedImage(bitmap, imagePath);
            bitmap = PictureUtils.getCompressedBitmap(bitmap, compressRatio);
            if(doWatermark) {
                bitmap = PictureUtils.watermarkImage(context, bitmap);
            }

            String path = PictureUtils.cacheImage(context, bitmap, imageName + System.currentTimeMillis() + ".jpeg");
            processedImagesBitmapPaths.add(path);
        }

        return processedImagesBitmapPaths;
    }

    public static Bitmap getRotatedImage(Bitmap bitmap, String path){
        ExifInterface exifInterface = null;
        try{
            exifInterface = new ExifInterface(path);
        } catch (IOException e){
            e.printStackTrace();
        }

        int orientation = exifInterface.getAttributeInt(ExifInterface.TAG_ORIENTATION, ExifInterface.ORIENTATION_UNDEFINED);
        Log.d("ORIENTATION", "getRotatedImage: " + orientation);
        Matrix matrix = new Matrix();
        switch (orientation) {
            case ExifInterface.ORIENTATION_NORMAL:
                return bitmap;
            case ExifInterface.ORIENTATION_FLIP_HORIZONTAL:
                matrix.setScale(-1, 1);
                break;
            case ExifInterface.ORIENTATION_ROTATE_180:
                matrix.setRotate(180);
                break;
            case ExifInterface.ORIENTATION_FLIP_VERTICAL:
                matrix.setRotate(180);
                matrix.postScale(-1, 1);
                break;
            case ExifInterface.ORIENTATION_TRANSPOSE:
                matrix.setRotate(90);
                matrix.postScale(-1, 1);
                break;
            case ExifInterface.ORIENTATION_ROTATE_90:
                matrix.setRotate(90);
                break;
            case ExifInterface.ORIENTATION_TRANSVERSE:
                matrix.setRotate(-90);
                matrix.postScale(-1, 1);
                break;
            case ExifInterface.ORIENTATION_ROTATE_270:
                matrix.setRotate(-90);
                break;
            default:
                return bitmap;
        }
        try {
            Bitmap bmRotated = Bitmap.createBitmap(bitmap, 0, 0, bitmap.getWidth(), bitmap.getHeight(), matrix, true);
            bitmap.recycle();
            return bmRotated;
        }
        catch (OutOfMemoryError e) {
            e.printStackTrace();
            return null;
        }
    }

//    public static double[] getImageGeolocation(String imagePath){
//        double[] latLonLocation = get
//        return null;
//    }

    public static double[] getImageLatLon(Context context, String path){
        String[] columns = new String[] {MediaStore.Images.Media.LONGITUDE, MediaStore.Images.Media.LATITUDE};

        path = StringUtility.extractImageNameFromPath(path);
        String queryStringArgs = "%"+path+"%";
        final Cursor cursor = context.getContentResolver()
                .query(MediaStore.Images.Media.EXTERNAL_CONTENT_URI
                        ,columns,
                        MediaStore.Images.Media.DATA + " like ?",
                        new String[] {queryStringArgs},
                        null);

        if(cursor.getCount() == 0){
            System.out.println("getCount null");
            return null;
        }

        int latColumnIndex = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.LATITUDE);
        int lonColumnIndex = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.LONGITUDE);

        double latValue = 0;
        double lonValue = 0;

        if(cursor.moveToFirst()){
            try{
                latValue = cursor.getDouble(latColumnIndex);
                lonValue = cursor.getDouble(lonColumnIndex);
            } finally {
                cursor.close();
            }
        } else {

        }
        if(latValue == 0 || lonValue == 0){
            return null;
        }

        return new double[]{latValue, lonValue};
    }
}
